      program TracerModel2D
c     --------- Parameters and Table Dimension Declaration ----------
      parameter(im=62,jm=46,pt=500)
c     Domain-related parameters (2D-tables)      
      real*8 alon(im,jm),alat(im,jm),h(im,jm),u(im,jm),v(im,jm),
     &fsm(im,jm),Kh(im,jm)
      real*8 dx,dy,horcon
c     Particle-related parameters
      real*8  xo(pt),yo(pt),zo(pt),x(pt),y(pt),z(pt),age(pt),mass(pt),
     &hp(pt)
      real*8 d50,ws,g,pi,tim,red,T50,mass0
      real*8 h1,h2,h3,h4,h5,h6,h7,h8,h9,r1,r2,r3,r4,r5,r6,r7,r8,r9,
     &u1,u2,u3,u4,u5,u6,u7,u8,u9,v1,v2,v3,v4,v5,v6,v7,v8,v9,
     &ed1,ed2,ed3,ed4,ed5,ed6,ed7,ed8,ed9,f1,f2,f3,f4,f5,f6,f7,f8,f9,
     &hh,ed,uu,vv,edmin
      integer ip(pt),nm,ndays,isource,iso(10),jso(10),pout(10),p,ptot
      character*1 ptype
c     ----------------------------------------------------------------     

c     _____________ Definition of simulation parameters ______________
      g=9.8065
      pi=3.1415926535897932384626433832795
      dt=20.0
      ndays=5.
      tim=0.0
      horcon=0.1
      edmin=5.
      nm=int(ndays*86400/dt)
      iso(1)=24;jso(1)=40
      iso(2)=13;jso(2)=34
      iso(3)=20;jso(3)=31
c     ----------------------------------------------------------------  

c     ****************************************************************      
c     Reading input data
c     Domain depths and seawater velocities      
      open(1,file='Bathymetry.dat')
      open(2,file='Horizontal_fields_25.dat')
      do l=1,im*jm
      read(1,*)i,j,alon(i,j),alat(i,j),dx,dy,h(i,j)
      read(2,*)i,j,alon(i,j),alat(i,j),u(i,j),v(i,j),zz
c     setting mask values: water cell --> fsm=1, land cell --> fsm=0
      if(h(i,j).gt.2)then
      h(i,j)=max(H(i,j),15.0)
      fsm(i,j)=1.0
      else
      fsm(i,j)=0.0
      endif
      enddo
      close(1);close(2)

      dx=1358.;dy=1779.
c     ________________ User-defined input parameters _________________
c     choosing the particle source
      write(*,*)'**************************************************'
54    write(*,*)'Select the location of the particle source'
      write(*,*)'by choosing the corresponding number from the list:'
      write(*,*)'Thessaloniki Bay --> 1'
      write(*,*)'Axios Delta      --> 2'
      write(*,*)'Nea Michaniona   --> 3'
      read(*,*)isource
      if(isource.eq.1)then
      is=iso(1);js=jso(1)
      elseif(isource.eq.2)then
      is=iso(2);js=jso(2)
      elseif(isource.eq.3)then
      is=iso(3);js=jso(3)
      else
      write(*,*)'Invalid source number. Please try again.'
      goto 54
      endif
c     choosing total number of particles for simulation 
      write(*,*)'**************************************************'
55    write(*,*)'How many particles enter at the source?'
      write(*,*)'suggested values between 5 and 500'
      read(*,*)ptot
      if(ptot.lt.5.or.ptot.gt.500)then
      write(*,*)'Invalid number of particles. Please try again.'
      goto 55
      endif
c     choosing the particle diameter and type
      write(*,*)'**************************************************'
56    write(*,*)'Give the value of the particle diameter in meters'
      write(*,*)'suggested values between 0.000001 and 0.001'
      write(*,*)'If you wish not to account for settling, select a value
     &less than 0.000001'
      read(*,*)d50
      if(d50.gt.0.0011)then
      write(*,*)'Invalid particle diameter. Please try again.'
      goto 56
      endif
      if(d50.lt.0.000001)then 
      ws=0.0
      else
      ws=((2000.-1024.)/1024.)*g*(d50**2.)/(18.*10.**(-3.))
      endif
      ws=min(ws,0.001)
      write(*,'(a30,x,f10.3,x,a5)')'Stokes settling velocity, ws=',ws,
     &'m/s'
      write(*,*)'**************************************************'
57    write(*,*)'Is the substance conservative?'
      write(*,*)'If yes write Y or y'
      write(*,*)'If no write N or n'
      read(*,*)ptype
      if(ptype=='Y'.or.ptype=='y')then
      red=0.0
      T50=0.0
      mass0=1.0
      write(*,*)'**************************************************'  
      write(*,*)'Default value for mass is taken into account'
      write(*,*)mass0      
      write(*,*)'press enter to proceed'
      pause
      elseif(ptype=='N'.or.ptype=='n')then
      write(*,*)'You have selected a non-conservative substance'
58    write(*,*)'Give the half-life value in days'
      read(*,*)red
      
      if(red.le.0.0)then
      write(*,*)'Invalid half-life. Please provide a positive, non-zero 
     &value.'
      goto 58
      else
      red=red*86400.
      T50=-log(0.5)/red
      endif
      write(*,*)'**************************************************'  
      write(*,*)'Give the particle mass'
      read(*,*)mass0      
      else
      write(*,*)'Invalid answer. Please try again.'
      goto 57
      endif

      write(*,*)'******************* Simulation begun ****************
     &*****'
      
      xo(1:ptot)=dx*(dfloat(is)-0.5) !x location of source
      yo(1:ptot)=dy*(dfloat(js)-0.5) !y location of source
      zo(1:ptot)=0.0 !z location of source
      mass(1:ptot)=mass0 !particle mass
      age(1:ptot)=0.0 !particle age
      ip(1:ptot)=0 !particle index for suspended particles
      write(*,*)'done array initialisation'
      
c     ****************************************************************

c     --------------------------- Main Program -----------------------     
c     Calculating Smagorinski diffusivity
      do i=2,im-1
      do j=2,jm-1
      Kh(i,j)=horcon*dx*dy*sqrt(((u(i+1,j)-u(i,j))/dx)**2.
     $+((v(i,j+1)-v(i,j))/dy)**2.+0.5*(0.25*(u(i,j+1)+u(i+1,j+1)
     $-u(i,j-1)-u(i+1,j-1))/dy+.25*(v(i+1,j)+v(i+1,j+1)
     $ -v(i-1,j)-v(i-1,j+1))/dx)**2.)*fsm(i,j)
      Kh(i,j)=max(Kh(i,j),edmin)
      enddo
      enddo
      
      do i=1,im
      Kh(i,1)=Kh(i,2)*fsm(i,1)
      Kh(i,jm)=Kh(i,jm-1)*fsm(i,jm)
      enddo
      do j=1,jm
      Kh(1,j)=Kh(2,j)*fsm(1,j)
      Kh(im,j)=Kh(im-1,j)*fsm(im,j)
      enddo
c     opening 5 files for output     
      nout=int(ptot/5)
      do ll=1,5
      pout(ll)=ll*nout
      open(ll,file='particle_'//char(ll+48)//'.txt')
      write(ll,'(a14,x,i10)')'Particle No:',pout(ll)
      write(ll,'(a7,5(x,a4))')'time[d]','x[m]','y[m]','z[m]','mass','ip'
      enddo
      
c     ---------------------- Temporal Loop --------------------------
      do n=1,nm
      tim=tim+dt !increase time 
      if(mod(tim,100.).eq.0)write(*,*)'time=',tim/86400.,'days'
c     ---------------------- Particle Loop --------------------------      
      do p=1,ptot
      if(ip(p).eq.0)then ! loop active only for particles in the water column
      age(p)=age(p)+dt !increasing particle age
      mass(p)=mass0*exp(-T50*tim)
      i=int(xo(p)/dx)+1;j=int(yo(p)/dy)+1
      
      if(j.eq.1)then
         h4=dabs(h(i-1,j))  ;f4=fsm(i-1,j)
         h5=dabs(h(i,j))    ;f5=fsm(i,j)
         h6=dabs(h(i+1,j))  ;f6=fsm(i+1,j)
         h7=dabs(h(i-1,j+1));f7=fsm(i-1,j+1)
         h8=dabs(h(i,j+1))  ;f8=fsm(i,j+1)
         h9=dabs(h(i+1,j+1));f9=fsm(i+1,j+1)      
         x1=dx*(dfloat(i-1)-0.5)
         x2=dx*(dfloat(i)-0.5);y2=dy*(dfloat(j)-0.5)
         x3=dx*(dfloat(i+1)-0.5);y3=dy*(dfloat(j+1)-0.5) 
         r4=sqrt((x1-xo(p))**2+(y2-yo(p))**2)
         r5=sqrt((x2-xo(p))**2+(y2-yo(p))**2)
         r6=sqrt((x3-xo(p))**2+(y2-yo(p))**2)
         r7=sqrt((x1-xo(p))**2+(y3-yo(p))**2)
         r8=sqrt((x2-xo(p))**2+(y3-yo(p))**2)
         r9=sqrt((x3-xo(p))**2+(y3-yo(p))**2)      
      ed4=Kh(i-1,j);ed5=Kh(i,j);ed6=Kh(i+1,j)
      ed7=Kh(i-1,j+1);ed8=Kh(i,j+1);ed9=Kh(i+1,j+1)

      u4=u(i-1,j);u5=u(i,j);u6=u(i+1,j)
      u7=u(i-1,j+1);u8=u(i,j+1);u9=u(i+1,j+1)
 
      v4=v(i-1,j);v5=v(i,j);v6=v(i+1,j)
      v7=v(i-1,j+1);v8=v(i,j+1);v9=v(i+1,j+1)  
      
      if(abs(r5).le.0.01)then
      hh=h5;ed=ed5;uu=u5;vv=v5
      goto 44
      endif
      hh=((f4*h4/r4+f5*h5/r5+f6*h6/r6+f7*h7/r7+
     &f8*h8/r8+f9*h9/r9)/(f4*1/r4+f5*1/r5+
     &f6*1/r6+f7*1/r7+f8*1/r8+f9*1/r9))
     
      ed=((f4*ed4/r4+f5*ed5/r5+f6*ed6/r6
     &+f7*ed7/r7+f8*ed8/r8+f9*ed9/r9)/(f4*1/r4
     &+f5*1/r5+f6*1/r6+f7*1/r7+f8*1/r8+f9*1/r9))
     

      uu=((f4*u4/r4+f5*u5/r5+f6*u6/r6
     &+f7*u7/r7+f8*u8/r8+f9*u9/r9)/(f4*1/r4
     &+f5*1/r5+f6*1/r6+f7*1/r7+f8*1/r8+f9*1/r9))
     

      vv=((f4*v4/r4+f5*v5/r5+f6*v6/r6
     &+f7*v7/r7+f8*v8/r8+f9*v9/r9)/(f4*1/r4
     &+f5*1/r5+f6*1/r6+f7*1/r7+f8*1/r8+f9*1/r9))
       goto 44                         
      endif
!      write(*,*)i,j,xo(p),yo(p)
!      pause
c     linear interpolation for depth, velocities, diffusivity     
         h1=dabs(h(i-1,j-1));f1=fsm(i-1,j-1)    
         h2=dabs(h(i,j-1))  ;f2=fsm(i,j-1)   
         h3=dabs(h(i+1,j+1));f3=fsm(i+1,j-1)
         h4=dabs(h(i-1,j))  ;f4=fsm(i-1,j)
         h5=dabs(h(i,j))    ;f5=fsm(i,j)
         h6=dabs(h(i+1,j))  ;f6=fsm(i+1,j)
         h7=dabs(h(i-1,j+1));f7=fsm(i-1,j+1)
         h8=dabs(h(i,j+1))  ;f8=fsm(i,j+1)
         h9=dabs(h(i+1,j+1));f9=fsm(i+1,j+1)
         x1=dx*(dfloat(i-1)-0.5);y1=dy*(dfloat(j-1)-0.5)
         x2=dx*(dfloat(i)-0.5);y2=dy*(dfloat(j)-0.5)
         x3=dx*(dfloat(i+1)-0.5);y3=dy*(dfloat(j+1)-0.5)
         r1=sqrt((x1-xo(p))**2+(y1-yo(p))**2)
         r2=sqrt((x2-xo(p))**2+(y1-yo(p))**2)
         r3=sqrt((x3-xo(p))**2+(y1-yo(p))**2)
         r4=sqrt((x1-xo(p))**2+(y2-yo(p))**2)
         r5=sqrt((x2-xo(p))**2+(y2-yo(p))**2)
         r6=sqrt((x3-xo(p))**2+(y2-yo(p))**2)
         r7=sqrt((x1-xo(p))**2+(y3-yo(p))**2)
         r8=sqrt((x2-xo(p))**2+(y3-yo(p))**2)
         r9=sqrt((x3-xo(p))**2+(y3-yo(p))**2)
         
      ed1=Kh(i-1,j-1);ed2=Kh(i,j-1);ed3=Kh(i+1,j-1)
      ed4=Kh(i-1,j);ed5=Kh(i,j);ed6=Kh(i+1,j)
      ed7=Kh(i-1,j+1);ed8=Kh(i,j+1);ed9=Kh(i+1,j+1)

      u1=u(i-1,j-1);u2=u(i,j-1);u3=u(i+1,j-1)
      u4=u(i-1,j);u5=u(i,j);u6=u(i+1,j)
      u7=u(i-1,j+1);u8=u(i,j+1);u9=u(i+1,j+1)
 
      v1=v(i-1,j-1);v2=v(i,j-1);v3=v(i+1,j-1)
      v4=v(i-1,j);v5=v(i,j);v6=v(i+1,j)
      v7=v(i-1,j+1);v8=v(i,j+1);v9=v(i+1,j+1)                    
!      write(*,*)'done calcs for interpolation'
C     Depths --> interpolations from the neihbouring grids
      if(abs(r5).le.0.01)then
      hh=h5;ed=ed5;uu=u5;vv=v5
      goto 44
      endif
	hh=((f1*h1/r1+f2*h2/r2+f3*h3/r3+f4*h4/r4+f5*h5/r5+f6*h6/r6+f7*h7/r7+
     &f8*h8/r8+f9*h9/r9)/(f1*1/r1+f2*1/r2+f3*1/r3+f4*1/r4+f5*1/r5+
     &f6*1/r6+f7*1/r7+f8*1/r8+f9*1/r9))
     

      ed=((f1*ed1/r1+f2*ed2/r2+f3*ed3/r3+f4*ed4/r4+f5*ed5/r5+f6*ed6/r6
     &+f7*ed7/r7+f8*ed8/r8+f9*ed9/r9)/(f1*1/r1+f2*1/r2+f3*1/r3+f4*1/r4
     &+f5*1/r5+f6*1/r6+f7*1/r7+f8*1/r8+f9*1/r9))
     

      uu=((f1*u1/r1+f2*u2/r2+f3*u3/r3+f4*u4/r4+f5*u5/r5+f6*u6/r6
     &+f7*u7/r7+f8*u8/r8+f9*u9/r9)/(f1*1/r1+f2*1/r2+f3*1/r3+f4*1/r4
     &+f5*1/r5+f6*1/r6+f7*1/r7+f8*1/r8+f9*1/r9))
     

      vv=((f1*v1/r1+f2*v2/r2+f3*v3/r3+f4*v4/r4+f5*v5/r5+f6*v6/r6
     &+f7*v7/r7+f8*v8/r8+f9*v9/r9)/(f1*1/r1+f2*1/r2+f3*1/r3+f4*1/r4
     &+f5*1/r5+f6*1/r6+f7*1/r7+f8*1/r8+f9*1/r9))
!44      write(*,*)h1,h2,h3,h4,h5,h6,h7,h8,h9,r1,r2,r3,r4,r5,r6,r7,r8,r9,
!     &u1,u2,u3,u4,u5,u6,u7,u8,u9,v1,v2,v3,v4,v5,v6,v7,v8,v9,
!     &ed1,ed2,ed3,ed4,ed5,ed6,ed7,ed8,ed9,f1,f2,f3,f4,f5,f6,f7,f8,f9,
!     &hh,ed,uu,vv
!      write(*,*)'done interpolation'
!      pause
44    call random(temp)
      x(p)=xo(p)+uu*dt+ed*(2.*temp-1.)*dt
      call random(temp)
      y(p)=yo(p)+vv*dt+ed*(2.*temp-1.)*dt
      z(p)=zo(p)+ws*dt
!      write(*,*)'x,y,z',x(p),y(p),z(p)
c     ________________Checking boundary conditions:__________________
      ii=int(x(p)/dx)+1;jj=int(y(p)/dy)+1; !new i and j coordinates
c     Open boundary condition
      if(y(p).le.dy/2)then !--> the particle has crossed the southern open boundary
      ip(p)=3
      do ll=1,5
      if(p.eq.pout(ll))write(ll,32)tim/86400.,x(p),y(p),z(p),mass(p),
     &ip(p)
      enddo
      goto 101
      endif
c     Land boundary conditions
      if(h(ii,jj).le.1.1)then !--> the particle has moved on dry land
      if((ii.ne.i).and.(jj.eq.j)) then !--> reflection in the x direction
	x(p)=xo(p)
	elseif((ii.eq.i).and.(jj.ne.j)) then !--> reflection in the y direction
	y(p)=yo(p)
	else !--> reflection in both (x,y) directions
	x(p)=xo(p)
      y(p)=yo(p)
	endif
	endif
c     Bottom conditions      
      if(z(p).ge.hh)then !the particle has deposited
      ip(p)=1
      endif
      
c     ______________ setting time step as past values _________________
      xo(p)=x(p);yo(p)=y(p);zo(p)=z(p)      
      do ll=1,5
      if(p.eq.pout(ll))write(ll,32)tim/86400.,x(p),y(p),-z(p),mass(p),
     &ip(p)
      enddo
 32   format(f8.6,4(x,f12.6),x,i5)
      endif
      
101   enddo !end of particle loop (do p=1,ptot)
      enddo !end of temporal loop (do n=1,nm)
c     ----------------------------------------------------------------      
      stop
      end
      
      
      